//
//  MINTLShadowScrollView.swift
//  ControlSikenjo
//
//  Created by narita on 2018/12/05.
//  Copyright © 2018 narita. All rights reserved.
//

import Cocoa

class MINTLShadowScrollView: NSScrollView
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    override init(frame frameRect: NSRect)
    {
        super.init(frame: frameRect)
        self.wantsLayer = true
        self.layerContentsRedrawPolicy = .onSetNeedsDisplay
        
        self.topSideLayer    = CALayer()
        self.bottomSideLayer = CALayer()
        self.rightSideLayer  = CALayer()
        self.leftSideLayer   = CALayer()
    }
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    required init?(coder: NSCoder)
    {
        super.init(coder: coder)
        self.wantsLayer = true
        self.layerContentsRedrawPolicy = .onSetNeedsDisplay
        
        self.topSideLayer    = CALayer()
        self.bottomSideLayer = CALayer()
        self.rightSideLayer  = CALayer()
        self.leftSideLayer   = CALayer()
    }
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    
    override func layout()
    {
        super.layout()
        self.needsDisplay = true
    }
    
    override func tile()
    {
        super.tile()
        self.needsDisplay = true
    }
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    override func updateLayer()
    {
        super.updateLayer()
        
        CATransaction.disableActions
        {
            let w = self.bounds.width
            let h = self.bounds.height
            let x = self.bounds.origin.x
            let y = self.bounds.origin.y
            
            // top ----------------------------------------------------------
            if self.hasTopSide
            {  self.topSideLayer!.frame = NSMakeRect(x, y - 20, w, 20) }
            else
            {  self.topSideLayer!.frame = NSMakeRect(x, y - 40, w, 20) }
            
            self.updateProperty(layer: self.topSideLayer!)
            
            // bottom ----------------------------------------------------------
            if self.hasBottomSide
            {  self.bottomSideLayer!.frame = NSMakeRect(x, y + h, w, 20) }
            else
            {  self.bottomSideLayer!.frame = NSMakeRect(x, y + h + 20, w, 20) }
            
            self.updateProperty(layer: self.bottomSideLayer!)

            // right ----------------------------------------------------------
            if self.hasRightSide
            {  self.rightSideLayer!.frame = NSMakeRect(x + w, y, 20, h) }
            else
            {  self.rightSideLayer!.frame = NSMakeRect(x + w + 20, y, 20, h) }
            
            self.updateProperty(layer: self.rightSideLayer!)

            
            // left ----------------------------------------------------------
            if self.hasLeftSide
            {  self.leftSideLayer!.frame = NSMakeRect(x - 20, y, 20, h) }
            else
            {  self.leftSideLayer!.frame = NSMakeRect(x - 40, y, 20, h) }
            
            self.updateProperty(layer: self.leftSideLayer!)
        }
    }

    private func updateProperty(layer inLayer: CALayer?)
    {
        if let theLayer = inLayer
        {
            theLayer.backgroundColor = NSColor.black.cgColor
            theLayer.shadowOffset    = NSMakeSize(0, 2)
            theLayer.shadowRadius    = 8.0
            theLayer.shadowColor     = NSColor.black.cgColor
            theLayer.shadowOpacity   = 1.0
            
            let theScrollerLayer = self.layer!.sublayers!.filter()
            {
                $0.delegate?.isKind(of: NSScroller.self) ?? false
            }.first
            
            theLayer.removeFromSuperlayer()
            self.layer!.insertSublayer(theLayer, below: theScrollerLayer)
        }
    }
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    override var wantsUpdateLayer: Bool
    {
        return true
    }
    
    @objc var topSideLayer    : CALayer? = nil
    @objc var bottomSideLayer : CALayer? = nil
    @objc var rightSideLayer  : CALayer? = nil
    @objc var leftSideLayer   : CALayer? = nil

    // 4辺のそれぞれに影をつけるか否か
    @objc var hasTopSide    : Bool = true
    {
        didSet
        {
            self.needsDisplay = true
        }
    }

    @objc var hasBottomSide : Bool = true
    {
        didSet
        {
            self.needsDisplay = true
        }
    }

    @objc var hasRightSide  : Bool = true
    {
        didSet
        {
            self.needsDisplay = true
        }
    }

    @objc var hasLeftSide   : Bool = true
    {
        didSet
        {
            self.needsDisplay = true
        }
    }


    // MARK: - Utility methods

    
}
